from peewee import Model, CharField, ForeignKeyField, IntegerField
from peewee import SqliteDatabase

DATABASE = "app.db"

database = SqliteDatabase(DATABASE)


# base model definition to use correct storage
class BaseModel(Model):
    class Meta:
        database = database


class Paper(BaseModel):
    journal = CharField()  # TODO seperate table for journals
    title = CharField()
    abstract = CharField()
    year = IntegerField()  # TODO DateTimeField() maybe?
    url = CharField()

    def authors(self):
        return Author.select().join(AuthorPaper).where(AuthorPaper.paper == self)

    def keywords(self):
        return Keyword.select().join(KeywordPaper).where(KeywordPaper.paper == self)

    def to_dict(self, exclude=[]):
        d = {
            field: getattr(self, field)
            for field in ["journal", "title", "abstract", "year", "url", "id"]
            if field not in exclude
        }
        if "authors" not in exclude:
            d["authors"] = [a.name for a in self.authors()]
        if "keywords" not in exclude:
            d["keywords"] = [k.name for k in self.keywords()]
        return d


class Author(BaseModel):
    name = CharField(unique=True)

    def papers(self):
        return Paper.select().join(AuthorPaper).where(AuthorPaper.author == self)


# relation table for many to many author to paper
class AuthorPaper(BaseModel):
    author = ForeignKeyField(Author)
    paper = ForeignKeyField(Paper)


class Keyword(BaseModel):
    name = CharField(unique=True)


# relation table for many to many keyword to paper
class KeywordPaper(BaseModel):
    keyword = ForeignKeyField(Keyword)
    paper = ForeignKeyField(Paper)


def create_tables():
    database.create_tables([Paper, Author, AuthorPaper, Keyword, KeywordPaper, Paper])


def load_df(df):
    for _, row in df.iterrows():
        paper = Paper.create(
            title=row["title"],
            journal=row["journal"],
            abstract=row["abstract"],
            year=row["year"],
            url=row["url"],
        )
        for author_name in row["authors"]:
            author, _ = Author.get_or_create(name=author_name)
            AuthorPaper.create(author=author, paper=paper)

        for keyword_name in row["keywords"]:
            keyword, _ = Keyword.get_or_create(name=keyword_name)
            KeywordPaper.create(keyword=keyword, paper=paper)


if __name__ == "__main__":
    import pandas as pd

    create_tables()
    for filename in ["jise", "isecon", "isedj"]:
        print(filename)
        df = pd.read_pickle(f"scraping/{filename}.pickle")
        load_df(df)
