from flask import Flask, jsonify, request
from .database import database, Author, Paper, KeywordPaper, Keyword, AuthorPaper
import networkx as nx
from networkx import node_link_data
import itertools
from peewee import fn
from .pdf import pdf_to_paper

app = Flask(__name__, static_url_path="")


# make sure database is open before request
@app.before_request
def _db_connect():
    database.connect()


# make sure database is closed after
@app.teardown_request
def _db_close(exc):
    if not database.is_closed():
        database.close()


@app.route("/api/paper", methods=["GET"])
def paper():
    search = request.args.get("query", default="")
    journals = request.args.get("journal").split(",")
    minPapers = request.args.get("minPapers")

    try:
        minPapers = int(minPapers)
    except ValueError:
        minPapers = 1

    title_search = Paper.select().where(Paper.title.contains(search))

    author_search = (
        Paper.select()
        .join(AuthorPaper)
        .join(Author)
        .where(Author.name.contains(search))
    )

    keyword_search = (
        Paper.select()
        .join(KeywordPaper)
        .join(Keyword)
        .where(Keyword.name.contains(search))
    )

    journal_match = (
        Paper.select()
        .where(Paper.journal.in_(journals))
        .join(AuthorPaper)
        .join(Author)
        .group_by(Author.id)
        .having(fn.COUNT(Author.id) >= minPapers)
    )

    query = title_search | author_search | keyword_search & journal_match

    papers = [p.to_dict(exclude=["abstract"]) for p in query]
    G = nx.Graph()
    for paper in papers:
        G.add_edges_from(itertools.combinations(paper["authors"], 2))

    k = nx.number_of_nodes(G)  # limit k for centrality check to save time
    print(G)
    if k > 2:
        if k > 256:
            k = 256
        bb = nx.betweenness_centrality(G, k)
        dd = nx.degree_centrality(G)
        ev = nx.eigenvector_centrality_numpy(G)

        nx.set_node_attributes(G, name="betweenness", values=bb)
        nx.set_node_attributes(G, name="degree", values=dd)
        nx.set_node_attributes(G, name="eigenvector", values=ev)

    return jsonify({"papers": papers, "graph": node_link_data(G)})


@app.route("/api/paper/<pid>", methods=["DELETE"])
def delete(pid):
    Paper.delete_by_id(pid)
    AuthorPaper.delete().where(AuthorPaper.paper.id == pid)
    return jsonify({})


@app.route("/api/paper", methods=["POST"])
def create():
    paper = Paper.create(**request.json)
    for author_name in request.json["authors"]:
        author, _ = Author.get_or_create(name=author_name)
        AuthorPaper.create(author=author, paper=paper)
    return jsonify(Paper.get_by_id(paper.id).to_dict())


@app.route("/api/author", methods=["GET"])
def authors():
    return jsonify([a.name for a in Author.select()])


@app.route("/api/upload", methods=["POST"])
def upload():

    fp = request.files["file"]
    try:
        paper = pdf_to_paper(fp)
    except Exception as e:
        print(e)
    return jsonify(paper)


@app.route("/", defaults={"path": ""})
@app.route("/<path:path>")
def index(path):
    return app.send_static_file("index.html")
