Vue.component("add-modal", {
  data() {
    return {
      paper: {
        journal: "",
        title: "",
        abstract: "",
        year: "",
        url: "",
        authors: []
      },
      authorText: ""
    };
  },
  methods: {
    onDrop: async function(e) {
      e.stopPropagation();
      e.preventDefault();
      const files = e.dataTransfer.files;
      const data = new FormData();
      data.append("file", files[0]);
      const response = await fetch("/api/upload", {
        method: "POST",
        body: data
      });
      const paper = await response.json();
      this.paper = paper;
    },
    add: async function() {
      // Trim and seperate author text
      this.paper.authors = this.authorText.split(",").map(function(author) {
        return author.trim();
      });
      const response = await fetch("/api/paper", {
        method: "POST",
        headers: {
          Accept: "application/json",
          "Content-Type": "application/json"
        },
        body: JSON.stringify(this.paper)
      });
      console.log(response);
    }
  },
  template: `
    <transition name="modal">
      <div class="modal-mask">
        <div class="modal-wrapper">
          <div class="modal-container" @drop="onDrop">
            <input type="text" v-model="paper.journal" placeholder="Journal"><br>
            <input type="text" v-model="paper.title" placeholder="Title"><br>
            <textarea v-model="paper.abstract" placeholder="Abstract"></textarea><br>
            <input type="number" v-model="paper.year" placeholder="Year"><br>
            <input type="text" v-model="paper.url" placeholder="URL"><br>
            <input type="text" v-model="authorText" placeholder="Author"><br>
            <button @click="add()">Add</button>
            <button @click="$emit('close')">Cancel</button>
          </div>
        </div>
      </div>
    </transition>
  `
});

Vue.component("network-graph", {
  props: ["nodes", "links", "focus", "colorBy"],
  data() {
    return {
      svg: null,
      zoom: null
    };
  },
  template: `
    <svg width="90%" height="600">
      <g id="container">
        <g stroke="#999" stroke-opacity="0.6">
          <line
            v-for="link in links"
            :x1="link.source.x"
            :y1="link.source.y"
            :x2="link.target.x"
            :y2="link.target.y"
          ></line>
        </g>
        <g stroke="#fff" stroke-width="1.5">
          <circle
            v-for="node in nodes"
            :fill="colorScale(node[colorBy])"
            :key="node.id"
            r="10"
            :cx="node.x"
            :cy="node.y"
          >
            <title>{{ node.id }}</title>
          </circle>
        </g>
      </g>
    </svg>`,
  mounted() {
    this.svg = d3.select("svg");
    container = d3.select("#container");
    this.zoom = d3
      .zoom()
      .scaleExtent([0.1, 10])
      .on("zoom", function() {
        container.attr("transform", d3.event.transform);
      });
    this.svg.call(this.zoom);
  },
  computed: {
    max: function() {
      var that = this;
      return Math.max.apply(
        Math,
        this.nodes.map(function(o) {
          return o[that.colorBy];
        })
      );
    }
  },
  watch: {
    focus: function(id) {
      node = this.nodes.find(n => {
        return n.id == id;
      });
      if (node) {
        const scale = 4;
        const translate = [
          this.getWidth() / 2 - scale * node.x,
          this.getHeight() / 2 - scale * node.y
        ];
        this.svg
          .transition()
          .duration(1000)
          .call(
            this.zoom.transform,
            d3.zoomIdentity.translate(translate[0], translate[1]).scale(scale)
          );
      }
    }
  },
  methods: {
    colorScale(value) {
      return d3
        .scaleLinear()
        .domain([0, this.max])
        .range(["blue", "red"])(value);
    },
    getWidth() {
      return this.svg.style("width").slice(0, -2);
    },
    getHeight() {
      return this.svg.style("height").slice(0, -2);
    }
  }
});

var app = new Vue({
  el: "#app",
  data() {
    return {
      add: false,
      colorBy: "betweenness",
      query: "",
      focused: "",
      journal: ["JISE", "ISEDJ"],
      papers: [],
      links: [],
      nodes: [],
      minPapers: 1,
      worker: null,
      progress: 0
    };
  },
  created() {
    this.fetchData();
  },
  methods: {
    async remove(id) {
      var url = new URL(`${window.location.origin}/api/paper/${id}`);
      const response = await fetch(url, { method: "DELETE" });
      if (response.ok) {
        this.papers = this.papers.filter(function(p) {
          return p.id !== id;
        });
      }
    },
    async fetchData() {
      var url = new URL(`${window.location.origin}/api/paper`);
      const params = {
        query: this.query,
        journal: this.journal,
        minPapers: this.minPapers
      };
      Object.keys(params).forEach(key =>
        url.searchParams.append(key, params[key])
      );
      const response = await fetch(url);
      const data = await response.json();
      this.papers = data.papers;
      const graph = data.graph;

      if (this.worker) {
        this.worker.terminate();
      }

      this.worker = new Worker("worker.js");

      this.worker.postMessage({
        nodes: graph.nodes,
        links: graph.links,
        height: 500,
        width: 500
      });

      this.worker.onmessage = function(event) {
        switch (event.data.type) {
          case "tick":
            return tick(event.data);
          case "end":
            return draw(event.data);
        }
      };

      var that = this;
      function tick(data) {
        that.progress = data.progress * 100;
      }

      function draw(data) {
        that.progress = 0;
        that.nodes = data.nodes;
        that.links = data.links;
      }
    }
  }
});

//  Prevent browser from trying to load pdfs
window.addEventListener(
  "dragover",
  function(e) {
    e = e || event;
    e.preventDefault();
  },
  false
);
window.addEventListener(
  "drop",
  function(e) {
    e = e || event;
    e.preventDefault();
  },
  false
);
