# Introduction

While a group of researchers will often be able to produce better work than that
of an individual researcher, finding suitable collaborators can be challenging.
Though there may be a large number of researchers in a given community, trying
to predict which potential collaboration will be effective can be a difficult
task. In communities such as Information Systems Education, collaboration often
spans across different geographic locations, which can further increase the
difficulty of the task.

By extracting publicly available co-authorship data from prominent organizations
in Information Systems Education and applying various machine learning
techniques, we can form predictions of future co-authorship.
These predictions may then serve as recommendations for suitable authors to
collaborate within the community.

# Related Work

Logistic Regression and Support Vector Machines have been used to predict
co-authorship among authors who are researching Coronary Artery Disease
[@yu2014predicting]. In another study, a similar technique  was performed on a
collection of Computer Science papers published between 1990 and 2003, in which
many models were used for link prediction on both an unweighted and weighted
co-authorship network. It found that, out of many models, a decision tree
classifier achieved the highest precision in predicting future co-authorship;
however, it was inconclusive whether assigning weights to edges based on author
contribution yielded improved results [@sa2010supervised]. On a study of link
prediction on the co-authorship network formed from the Institute of Electronics
Information and Communications Engineers (IEICE), a fitted SVM assigned low
weights to the shortest path, Jaccard's coefficient, PageRank, and SimRank
measures. This suggests that they may not be a good indicator of future
co-authorship. Conversely, higher weights were assigned to the Katz and
preferential attachment measures [@pavlov2007finding]. 

# Methodology

## Supervised Learning for Link Prediction

A supervised learning model is built to map an input of similarity measures to
probability of future collaboration. A data set of papers is split into three
consecutive time frames, and, for each time frame, a network graph is formed.
Each author is represented as a node, with edges connecting authors that have
collaborated in that time frame. A training set is then created by selecting
pairs of authors within the first time frame, computing the below similarity
measures, and labeling whether or not the pair collaborate in the next time
frame. A cross-validation set is created with the same method using the second
and third time-frame. Finally, we fit a model to the training set and measure
its accuracy on the cross-validation set.

## Similarity Measures

While many similarity measures can be computed for a pair of
nodes in a network graph, we selected the metrics listed in [@sa2010supervised]
with the addition of the Resource Allocation Algorithm, because it may be a
better measure for link prediction [@zhou2009predicting]. These measures were
computed with the help of NetworkX 2.2 [@hagberg2008exploring].

### Common Neighbors

Common neighbors are the number of coauthors that each author share.
The number of common neighbors between authors $u$ and $v$ can be written as:

$$ | N(u) \cap N(v) | $$

Where $N(u)$ denotes the set of all neighbors of node $u$.

### Jaccard Coefficient

The Jaccard Coefficient, originally *coefficient de communauté*
[@jaccard1908nouvelles], measures the similarity between two sets by dividing
the size of their intersection by the size of their union. We can compute the
Jaccard Coefficient of the neighbors of authors $u$ and $v$ like so:

$$\frac{| N(u) \cap N(v) |}{| N(u) \cup N(v) | }$$

Two authors that share no common neighbors will have a coefficient of 0, while
two authors that share the same set of neighbors will have a coefficient
of 1.

### Resource Allocation Algorithm

First introduced in [@zhou2009predicting], the Resource Allocation Algorithm is
defined as:

$$\sum_{w \in N(u) \cap N(v)} \frac{1}{|N(w)|}$$

If many of the common neighbors between $u$ and $v$ have a low number of
neighbors themselves, any "resources" sent from $u$ have a high likelihood of
making their way to $v$ and vice versa. In other words, the algorithm puts a
weight on how important the two authors are to the authors that they share.

### Preferential Attachment

Preferential attachment is the product of the size of each node's neighbor set:

$$ |N(u)||N(v)| $$

Two authors that both have a large number of collaborators will have a
large preferential attachment, which may indicate a higher probability of future
collaboration [@pavlov2007finding]

## Learning Methods

With a dataset labeled with the above features, we built a classification model
to predict the probability of future collaboration between two authors.
Logistic Regression is the most widely used for classification problems.
Support Vector Machines (SVM) may perform differently because it finds the
optimal hyperplane separating the classes [@cortes1995support]. A decision tree
has also shown to work well for link prediction [@sa2010supervised], so we chose to also
use a Random Forest model, an ensemble of decision trees.

### Logistic Regression

Since author pairs are classified as either future coauthors or not we use
a binomial model. Given label $y$, features $x$ and parameters $\theta$, we
write out hypothesis function like so:

$$ h_{\theta}(x) = \frac{1} {1 + e^{-\theta^Tx}}$$

The model can then be fit by minimizing cross-entropy loss function $J$:

$$J(\theta) = - \frac{1}{m}  \sum^m_{i=1}
[y^{(i)}\log(h_\theta(x^{(i)})) + (1 - y^{(i)})\log(1-h_\theta(x^{(i)}))] + 
\frac{\lambda}{2m} \sum\nolimits_j \theta_j^2 $$

$J(\theta)$ can be minimized via gradient descent.  Parameters $\theta$ can then
be viewed to obtain feature importance. A comparatively large positive parameter
indicates that its corresponding feature is more important in predicting a
positive label.  Conversely, a small parameter indicates that its corresponding
feature is less important in predicting future collaboration. $\lambda$ is
used for regularization to address any multicollinearity.

### Support Vector Machines

SVM function in a similar way to Logistic Regression, however, the hypothesis
function is written as:

$$ h_{\theta}(x) = 
\begin{cases} 
1 & \text{if } \theta^Tx\geq 0 \\ 0 & \text{otherwise} 
\end{cases}
$$

The model is then fit by minimizing the hinge loss function $J(\theta)$ with
gradient descent:

$$J(\theta) = C\sum\nolimits_i \max(0, 1-y^{(i)}\theta^Tx^{(i)}) +
\frac{1}{2}\sum\nolimits_j \theta_j^2$$

Unlike the cross-entropy loss function, the hinge loss function penalizes even a
correct classification if its prediction does not exceed the classification by
some margin. The result of minimizing the hinge loss is a hyperplane that
separates the classes of data with an optimal margin [@cortes1995support].
Similar to $\lambda$ in the cross-entropy loss function, $C$ is used for
regularization.

#### The Kernel Trick 

In some cases, samples cannot be linearly separated. A solution to this is by
applying kernels to the data, which maps the data into a higher-dimensional
space. This transformation allows the data to be separated with a linear
hyperplane, improving the classification ability of the model. For our model, we
will be using the Radial Basis Function kernel, which maps data into an
infinite-dimensional space, and is commonly used with SVMs
[@chang2010training].

### Random Forest

Sa et al. showed that decision trees can be used for link prediction with higher
accuracy than other models [@sa2010supervised]. As a single decision tree can
easily overfit training data, we use a Random Forest model instead. Each tree is
formed with a bootstrapped sample of the training set, so each tree is fit to a
different subset of the data. By building many "de-correlated" trees, and then
averaging the predictions, models generally perform with higher accuracy and
lower variance [@hastie2005elements].

With a fitted random forest, feature importance can be obtained using mean
decrease impurity; that is how much each feature's splits increase the purity of
the classification. We use the Random Forest classifier implemented in
scikit-learn 0.20.3 [@scikit-learn], which uses the Gini Index as a measure of purity. 

# Data

To obtain a dataset of authors in the IS education communities,
we collected papers and authors from the following journals and conferences:
Information Systems Education Conference (ISECON), EDSIG Conference on
Information Systems and Computing Education (EDSIGCON), Journal of Information
Systems Education (JISE), and Information Systems Education Journal (ISEDJ).

## Collection

For each organization, we developed a web scraping tool to extract author names,
paper names, and year from the organizations' website. Since each organization
has a differently structured website (some changing in structure over different
years), special care was taken to ensure that the data was extracted correctly.
It is worth noting, however, that some authors were listed with different
spellings of their names in different organizations, resulting in multiple
authors being extracted instead of one. The following URLs were used as a
starting point for the web scraper:

|Organization | URL|
 --- | ---
| ISECON 2000-2009 | http://proc.edsig.org/xref/title.html |
| ISECON 2010-2014 | http://proc.edsig.org/{year}/index.html |
| ISECON 2015-2018 | http://proceedings.isecon.org/dt-row-data |
| EDSIGCON 2015-2018 | http://proc.iscap.info/{year}/ |
| JISE | http://jise.org/archives.html |
| ISEDJ | http://isedj.org/archives.html |

Table: Data sources for each organization. 

In the case of ISECON from 2000 to 2014 and EDSIGCON 2015-2018, paper titles
were extracted from the "Titles" page of the respective year, and then
corresponding authors were extracted from the "Authors" page. ISECON paper
metadata from 2015 to 2018 was downloaded as a single JSON document from the above
URL. JISE and ISEDJ required programmatically navigating through each journals'
volumes, and then each paper within the volumes to extract the relevant
information. After all of the titles, authors, and years of each organization's
papers were extracted, the data was stored in an SQLite database. A total of 3522
papers from years 2000 to 2018 along with 3076 unique authors were extracted.
Although some papers from conferences and their corresponding journals may
contain duplicates, we did not assign any weight to the number of times two coauthors
collaborated, so the resulting network graph was not affected.

## Visualization

Once the data was collected, we began to analyze the social network graph formed
by co-authorship. To promote a better understanding of this social
network, we built a web application that allowed querying of the dataset and
interactive exploration of the network graph, which can be seen in Figure
\ref{webapp}.

![Screenshot of application.\label{webapp}](figures/webapp.png){height=400px}

## Preprocessing

In order to fit our collaboration prediction models, our dataset was split into
three time frames of equal length: $T_1 = [2000-2005]$, $T_2 = [2006-2011]$, and
$T_3 = [2012-2017]$. A training set was formed consisting of author pairs who
were active in both $T_1$ and $T_2$, *did not* collaborate in $T_1$, and *did*
collaborate in $T_2$. These pairs were labeled **positive**. Additionly, an
equal number of author pairs who were active in both $T_1$ and $T_2$, but did
*not* collaborate in either $T_1$ or $T_2$ were randomly sampled and labelled
**negative**. The same procedure was followed to create a cross-validation set
over $T_2$ and $T_3$. The resulting training set consisted of 370 samples, while
the cross-validation set contained 106 pairs.

The above-listed similarity measures were then computed for each pair over the
network graphs formed in $T_1$ and $T_2$ for the training set and validation
set, respectively. It is recommended to standardize input features before
training [@hastie2005elements], which was done by subtracting the mean, then
dividing by the standard deviation for each feature.

# Results

The training set was then fit to Logistic Regression, Support Vector Machine,
and Random Forest models, with accuracy measured on the cross-validation set. We
also measured each model's receiver operator characteristic (ROC) curve, which
shows the relationship of true positives vs. false positives as the decision
threshold is varied. The area under this curve (AUC) is then used to compare the
different models' classification ability.

Model | Accuracy | AUC
--- | --- | --- 
LR | 0.806 | 0.870
SVM | **0.827** | **0.871** 
RF | **0.827** | 0.844

Table: Accuracy and AUC for each learning method.

![ROC curves for each model.](figures/roc.png){height=256px}

We found the SVM and RF models to have the highest accuracy, being able to
predict future collaboration with an 82.7% accuracy (compared to 50% randomly
guessing). To see which similarity measures are more indicative of future
collaboration, we plotted the feature importance of both the LR and RF models, seen
in Figure \ref{feat}.

![Feature importance for LR and RF
models.\label{feat}](figures/feat.png){height=256px}

# Discussion 

All three models were able to predict future co-authorship quite well, using
only topological features from the co-authorship network. As has
been shown in past literature [@pavlov2007finding], preferential attachment is
a strong indicator of future co-authorship. That is, in a relatively small
community such as that of Information System Education, the greater the number of
coauthors two individuals have, the more likely they are to work together in the
future.

# Conclusion

Such a model may be used as a recommendation system to suggest future coauthors
for an individual by listing the authors in which he or she has the highest
probability of working with. Results may be improved by adding more features
such as commonality of keywords or language used in papers. 

# References
