import requests
from bs4 import BeautifulSoup
from collections import defaultdict
import re
from core import Paper, format_name, format_title
import pandas as pd

import requests_cache

# optional but prevents from redownloading all the pages during developement
requests_cache.install_cache("html_cache")


papers = []

# NOTE: the urls do not match the name, but they are still correct ¯\_(ツ)_/¯
edsig_base = "http://proc.edsig.org/"


# ISECON from 2000 - 2009
papers_html = requests.get(f"{edsig_base}xref/title.html").content
authors_html = requests.get("http://proc.edsig.org/xref/a1.html").content
papers_soup = BeautifulSoup(papers_html, features="lxml")
authors_soup = BeautifulSoup(authors_html, features="lxml")

authorship_table = defaultdict(list)

for name in authors_soup.find_all("a", href=False):  # authors are all anchors
    if name.parent.name != "h2":  # ignore headers
        sib = name.next_sibling
        while sib and sib.name != "br":
            if sib.name == "a":
                authorship_table[sib.text].append(format_name(name.text))
            sib = sib.next_sibling

for table in papers_soup.find_all("table")[2:]:
    for tr in table.find_all("tr"):
        tds = tr.find_all("td")
        link = tds[0].find("a")
        paper_num = link.text[5:]
        year = link.text[:4]
        url = edsig_base + link.get("href")[3:]
        title = format_title(tds[1].text)
        authors = authorship_table[paper_num]
        papers.append(Paper(title, "ISECON", "", year, authors, [], url))


# ISECON from 2010 - 2014
for year in range(2010, 2015):
    papers_html = requests.get(f"{edsig_base}{year}/index.html").content
    authors_html = requests.get(f"{edsig_base}{year}/authors.html").content

    papers_soup = BeautifulSoup(papers_html, features="lxml")
    authors_soup = BeautifulSoup(authors_html, features="lxml")

    authorship_table = defaultdict(list)  # { paper_number: list of authors}

    # fill authorship table with data from the authors page
    for sub_table in authors_soup.find("table").find_all("table"):
        for row in sub_table.find_all("td"):
            spans = row.find_all("span")
            if year == 2013 and spans[0].find("br"):  # weird formatting problem in 2013
                name = spans[2].text[:-2]  # trim " -" from end
                numbers = [s.text.strip("()") for s in spans[3:]]
            elif len(spans) == 2:
                name = spans[0].text[:-2]  # trim " -" from end
                numbers = spans[1].text.strip("()")
            else:
                name = spans[1].text[:-2]  # trim " -" from end
                numbers = [s.text.strip("()") for s in spans[2:]]
            for number in numbers:
                authorship_table[number].append(format_name(name))

    # generate papers
    for row in papers_soup.find("table").find_all("tr"):
        if len(row.find_all("span")) > 1:
            title = row.find_all("span")[1].text
            title = re.sub(r"\r\n", "", title).rstrip(",")
            title = format_title(title)
            link = row.find("a")
            number = link.text.strip("()")
            url = link.get("href")
            authors = authorship_table[number]
            papers.append(Paper(title, "ISECON", "", year, authors, [], url))


# ISECON from 2015 - 2018
data = requests.get("http://proceedings.isecon.org/dt-row-data").json()["data"]
for article in data:
    title = article["artTitle"]
    year = article["artDateTime"].split(", ")[-1][:4]  # just year for now
    abstract = article["artAbstract"]
    keywords = []
    url = ""  # TODO
    authors = [author["auFullName"] for author in article["authors"]]
    papers.append(Paper(title, "ISECON", abstract, year, authors, keywords, url))

# EDSIG from 2015 - 2018
for year in range(2015, 2019):
    papers_html = requests.get(f"http://proc.iscap.info/{year}/").content
    authors_html = requests.get(f"http://proc.iscap.info/{year}/authors.html").content

    papers_soup = BeautifulSoup(papers_html, features="lxml")
    authors_soup = BeautifulSoup(authors_html, features="lxml")

    authorship_table = defaultdict(list)  # { paper_number: list of authors}

    # fill authorship table with data from the authors page
    for sub_table in authors_soup.find("table").find_all("table"):
        for row in sub_table.find_all("td"):
            spans = row.find_all("span")
            if len(spans) == 2:
                name = spans[0].text[:-2]  # trim " -" from end
                numbers = spans[1].text.strip("()")
            else:
                name = spans[1].text[:-2]  # trim " -" from end
                numbers = [s.text.strip("()") for s in spans[2:]]
            for number in numbers:
                authorship_table[number].append(format_name(name))

    # generate papers
    for row in papers_soup.find("table").find_all("tr"):
        if len(row.find_all("span")) > 1:
            title = row.find_all("span")[1].text
            title = re.sub(r"\r\n", "", title).rstrip(",")
            title = format_title(title)
            link = row.find("a")
            number = link.text.strip(" ()")
            url = link.get("href")
            authors = authorship_table[number]
            papers.append(Paper(title, "EDSIG", "", year, authors, [], url))


df = pd.DataFrame.from_records(papers, columns=Paper._fields)
df.to_pickle("isecon.pickle")
