import pandas as pd
import pickle
import matplotlib.pyplot as plt
import networkx as nx
import itertools
import random
import pickle
import seaborn as sns
import numpy as np

np.random.seed(0)
random.seed(0)
sns.set()
sns.set_context("paper")
sns.set(font="serif")
sns.set_style("ticks", {"font.family": "cmr10"})
sns.set_palette("Dark2")
palette = sns.color_palette("Dark2")


def data_to_graphs(df, time_ranges):
    graphs = [nx.Graph() for _ in time_ranges]
    for row in df.itertuples():
        for t, time_range in enumerate(time_ranges):
            if row.year >= time_range[0] and row.year < time_range[1]:
                graphs[t].add_edges_from(itertools.combinations(row.authors, 2))
    return graphs


def positive_pairs(graph, next_graph):
    pairs = set()
    for pair in next_graph.edges:
        if (
            graph.has_node(pair[0])
            and graph.has_node(pair[1])
            and not graph.has_edge(*pair)
        ):
            pairs.add(pair)
    return pairs


def negative_pairs(graph, next_graph, n):
    pairs = set()
    while len(pairs) < n:
        pair = tuple(random.sample(graph.nodes, 2))
        if not graph.has_edge(*pair) and not next_graph.has_edge(*pair):
            pairs.add(pair)
    return pairs


def generate_features(graph, pair, label):
    return {
        "common_neighbors": len(list(nx.common_neighbors(graph, pair[0], pair[1]))),
        "jaccard_coefficient": list(nx.jaccard_coefficient(graph, [pair]))[0][2],
        "resource_allocation": list(nx.resource_allocation_index(graph, [pair]))[0][2],
        "preferential_attachment": list(nx.preferential_attachment(graph, [pair]))[0][
            2
        ],
        "label": label,
    }


def generate_set(graph, next_graph):
    positives = [
        generate_features(graph, pair, 1) for pair in positive_pairs(graph, next_graph)
    ]
    negatives = [
        generate_features(graph, pair, 0)
        for pair in negative_pairs(graph, next_graph, len(positives))
    ]
    return pd.DataFrame(positives + negatives)


jise = pd.read_pickle("scraping/jise.pickle")
isecon = pd.read_pickle("scraping/isecon.pickle")
isedj = pd.read_pickle("scraping/isedj.pickle")

df = pd.concat([jise, isecon, isedj])
df.year = df.year.astype(int)
print(f"{df.shape[0]} Papers")
total_authors = set()
for authors in df["authors"]:
    total_authors.update(authors)
print(f"{len(total_authors)} Unique Authors")


df["count"] = df["authors"].apply(len)
# remove giant panel things
df = df[df["count"] < 10]
time_ranges = [(2000, 2006), (2006, 2012), (2012, 2018)]


graphs = data_to_graphs(df, time_ranges)
df_train = generate_set(graphs[0], graphs[1])
df_test = generate_set(graphs[1], graphs[2])
X_train, y_train = df_train.drop(columns=["label"]), df_train["label"]
X_test, y_test = df_test.drop(columns=["label"]), df_test["label"]
mean = X_train.mean()
std = X_train.std()

X_train = (X_train - mean) / std
X_test = (X_test - mean) / std

print(f"Corr: {X_train.corr()}")

pickle.dump(std, open("models/std.pickle", "wb"))
pickle.dump(mean, open("models/mean.pickle", "wb"))

from sklearn.linear_model import LogisticRegression

lr = LogisticRegression(solver="lbfgs")
lr.fit(X_train, y_train)
print(f"LR Test Accuracy: {lr.score(X_test, y_test):.4f}")
pickle.dump(lr, open("models/lr.pickle", "wb"))

from sklearn.svm import SVC

svm = SVC(gamma="auto", kernel="rbf", C=1e1)
svm.fit(X_train, y_train)
print(f"SVM Test Accuracy: {svm.score(X_test, y_test):.4f}")
pickle.dump(svm, open("models/svm.pickle", "wb"))

from sklearn.ensemble import RandomForestClassifier

rf = RandomForestClassifier(n_estimators=100, max_depth=8)
rf.fit(X_train, y_train)
print(f"Random Forest Train Accuracy: {rf.score(X_train, y_train):.4f}")
print(f"Random Forest Test Accuracy: {rf.score(X_test, y_test):.4f}")
pickle.dump(rf, open("models/random_forest.pickle", "wb"))

from sklearn.metrics import roc_curve, auc

models = [lr, svm, rf]
names = ["LR", "SVM", "RF"]
fprs = []
tprs = []
for model in models:
    try:
        y_score = model.decision_function(X_test)
    except Exception:
        y_score = model.predict_proba(X_test)[:, 1]
    fpr, tpr, _ = roc_curve(y_test, y_score)
    fprs.append(fpr)
    tprs.append(tpr)

plt.plot([0, 1], [0, 1], linestyle="--", c="k")
for fpr, tpr, name in zip(fprs, tprs, names):
    plt.plot(fpr, tpr, label=f"{name} (area = {auc(fpr, tpr):.3f})")
plt.xlabel("False Positive Rate")
plt.ylabel("True Positive Rate")
plt.legend()
plt.savefig("roc.png", dpi=300)
plt.show()


def norm(x):
    return x / np.sum(x)


plt.figure(figsize=(8, 4))
ind = np.arange(len(lr.coef_[0]))
width = 0.35
plt.bar(ind, norm(lr.coef_[0]), width, label="LR", color=palette[0])
plt.bar(ind + width, norm(rf.feature_importances_), width, label="RF", color=palette[2])
plt.xticks(ind + width / 2, [col.replace("_", " ").title() for col in X_test.columns])
plt.legend()
plt.tight_layout()
plt.savefig("feat.png", dpi=300)
plt.show()
